package com.jiuqi.syntax;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.Stack;

public final class NiporlanParser {
   public static int NEW_COUNT = 0;
   private static final int ASSIGN_CELL = 0;
   private Stack stack = new Stack();
   private SyntaxParser syntaxParser;
   private SyntaxEvent syntaxEvent = new SyntaxEvent(this);
   private CommonData cellValue = new CommonData();
   private boolean useDataLocator = false;
   private ArrayList usedNiporlanItems = new ArrayList();
   private ArrayList unUsedNiporlanItems = new ArrayList();

   public NiporlanParser(SyntaxParser syntaxParser) {
      ++NEW_COUNT;
      this.syntaxParser = syntaxParser;
   }

   public boolean getUseDataLocator() {
      return this.useDataLocator;
   }

   public void setUseDataLocator(boolean value) {
      this.useDataLocator = value;
   }

   private NiporlanItem newNiporlanItem() {
      NiporlanItem niporlanItem;
      if (this.unUsedNiporlanItems.size() > 0) {
         niporlanItem = (NiporlanItem)this.unUsedNiporlanItems.get(this.unUsedNiporlanItems.size() - 1);
         this.unUsedNiporlanItems.remove(niporlanItem);
         this.usedNiporlanItems.add(niporlanItem);
      } else {
         niporlanItem = new NiporlanItem();
         this.usedNiporlanItems.add(niporlanItem);
      }

      return niporlanItem;
   }

   private void disposeNiporlanItem(NiporlanItem niporlanItem) {
      int index = this.usedNiporlanItems.indexOf(niporlanItem);
      if (index >= 0) {
         this.usedNiporlanItems.remove(index);
         this.unUsedNiporlanItems.add(niporlanItem);
      }

   }

   private void disposeAllNiporlanItem() {
      this.unUsedNiporlanItems.addAll(this.usedNiporlanItems);
      this.usedNiporlanItems.clear();
   }

   public boolean judge(Niporlan niporlan) {
      if (niporlan.isEmpty()) {
         return true;
      } else if (niporlan.ifExist()) {
         this.kernel(niporlan.ifItems, false);
         if (((NiporlanItem)this.stack.get(0)).getCommonData().cdBool) {
            this.kernel(niporlan.thenItems, false);
         } else if (niporlan.elseItems != null) {
            this.kernel(niporlan.elseItems, false);
         } else {
            this.stack.clear();
         }

         return this.stack.size() == 0 ? true : ((NiporlanItem)this.stack.get(0)).getCommonData().cdBool;
      } else {
         this.kernel(niporlan.thenItems, false);
         return this.stack.size() == 0 ? true : ((NiporlanItem)this.stack.get(0)).getCommonData().cdBool;
      }
   }

   public void evaluate(Niporlan niporlan) {
      if (!niporlan.isEmpty()) {
         if (niporlan.ifExist()) {
            this.kernel(niporlan.ifItems, false);
            if (((NiporlanItem)this.stack.get(0)).getCommonData().cdBool) {
               this.kernel(niporlan.thenItems, true);
            } else if (niporlan.elseItems != null) {
               this.kernel(niporlan.elseItems, true);
            } else {
               this.stack.clear();
            }
         } else {
            this.kernel(niporlan.thenItems, true);
         }

      }
   }

   public void attract(Niporlan niporlan, CommonData resultData) {
      if (niporlan.isEmpty()) {
         resultData.type = 0;
         resultData.cdReal = 0.0;
      } else {
         if (niporlan.ifExist()) {
            this.kernel(niporlan.ifItems, false);
            if (((NiporlanItem)this.stack.get(0)).getCommonData().cdBool) {
               this.kernel(niporlan.thenItems, false);
            } else if (niporlan.elseItems != null) {
               this.kernel(niporlan.elseItems, false);
            } else {
               this.stack.clear();
            }

            if (this.stack.size() == 0) {
               resultData.type = 0;
            } else {
               resultData.assign(((NiporlanItem)this.stack.get(0)).getCommonData());
            }
         } else {
            if (niporlan.thenItems.size() == 1) {
               NiporlanItem ni = (NiporlanItem)niporlan.thenItems.get(0);
               if (ni.getType() == 3) {
                  this.syntaxEvent.dataCell = ni.getDataCell();
                  this.syntaxEvent.value = this.cellValue;
                  if (this.useDataLocator) {
                     this.getCellValue(this.syntaxEvent, ni);
                  } else {
                     this.getCellValue(this.syntaxEvent);
                  }

                  resultData.assign(this.syntaxEvent.value);
                  if (resultData.type == 1) {
                     resultData.type = 2;
                     resultData.cdReal = (double)resultData.cdInt;
                  }

                  if (resultData.type == 0) {
                     this.error("Niporlan公式有错误(空数据类型)！");
                  }

                  return;
               }
            }

            this.kernel(niporlan.thenItems, false);
            if (this.stack.size() == 0) {
               resultData.type = 0;
            } else {
               resultData.assign(((NiporlanItem)this.stack.get(0)).getCommonData());
            }
         }

      }
   }

   private void bool2Real(CommonData data) {
      if (data.type == 5) {
         data.type = 2;
         if (data.cdBool) {
            data.cdReal = -1.0;
         } else {
            data.cdReal = 0.0;
         }
      }

   }

   private void real2Decimal(CommonData data) {
      if (data.type == 2) {
         data.type = 10;
         data.cdDecimal = new BigDecimal(data.cdReal);
      }

   }

   private void kernel(ArrayList niItems, boolean isAssignValue) {
      int assignKind = false;
      this.stack.clear();
      this.disposeAllNiporlanItem();
      int i = 0;

      for(int n = niItems.size(); i < n; ++i) {
         NiporlanItem item = (NiporlanItem)niItems.get(i);
         NiporlanItem rightItem;
         NiporlanItem leftItem;
         CommonData leftItemCommonData;
         switch (item.getType()) {
            case 0:
               CommonData rightItemCommonData;
               switch (item.getOp()) {
                  case 1:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     this.bool2Real(rightItemCommonData);
                     this.bool2Real(leftItemCommonData);
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     switch (leftItemCommonData.type) {
                        case 2:
                           leftItemCommonData.cdReal += rightItemCommonData.cdReal;
                           break;
                        case 3:
                           leftItemCommonData.cdString = leftItemCommonData.cdString + rightItemCommonData.cdString;
                           break;
                        case 10:
                           this.real2Decimal(rightItemCommonData);
                           if (leftItemCommonData.isNull) {
                              leftItemCommonData.cdDecimal = rightItemCommonData.cdDecimal;
                           } else if (!rightItemCommonData.isNull) {
                              leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.add(rightItemCommonData.cdDecimal);
                           }
                     }

                     leftItemCommonData.isNull = leftItemCommonData.isNull && rightItemCommonData.isNull;
                     leftItemCommonData.initNullValue();
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 2:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     this.bool2Real(rightItemCommonData);
                     this.bool2Real(leftItemCommonData);
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     switch (leftItemCommonData.type) {
                        case 2:
                           leftItemCommonData.cdReal -= rightItemCommonData.cdReal;
                           if (Math.abs(leftItemCommonData.cdReal) < 1.0E-5) {
                              leftItemCommonData.cdReal = 0.0;
                           }
                           break;
                        case 6:
                           leftItemCommonData.type = 2;
                           leftItemCommonData.cdReal = (double)SyntaxParser.minusDate(leftItemCommonData.cdDate, rightItemCommonData.cdDate);
                           if (Math.abs(leftItemCommonData.cdReal) < 1.0E-5) {
                              leftItemCommonData.cdReal = 0.0;
                           }
                           break;
                        case 10:
                           this.real2Decimal(rightItemCommonData);
                           if (leftItemCommonData.isNull) {
                              leftItemCommonData.cdDecimal = rightItemCommonData.cdDecimal.negate();
                           } else if (!rightItemCommonData.isNull) {
                              leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.subtract(rightItemCommonData.cdDecimal);
                           }
                     }

                     leftItemCommonData.isNull = leftItemCommonData.isNull && rightItemCommonData.isNull;
                     leftItemCommonData.initNullValue();
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 3:
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     this.bool2Real(leftItemCommonData);
                     if (leftItemCommonData.type == 10) {
                        if (!leftItemCommonData.isNull) {
                           leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.negate();
                        }
                     } else {
                        leftItem.getCommonData().cdReal = -leftItem.getCommonData().cdReal;
                     }

                     leftItem.getCommonData().initNullValue();
                     continue;
                  case 4:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     this.bool2Real(rightItemCommonData);
                     this.bool2Real(leftItemCommonData);
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     if (leftItemCommonData.type == 10) {
                        if (!leftItemCommonData.isNull && !rightItemCommonData.isNull) {
                           this.real2Decimal(rightItemCommonData);
                           leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.multiply(rightItemCommonData.cdDecimal);
                        }
                     } else {
                        leftItemCommonData.cdReal *= rightItemCommonData.cdReal;
                     }

                     leftItemCommonData.isNull = leftItemCommonData.isNull || rightItemCommonData.isNull;
                     leftItemCommonData.initNullValue();
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 5:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     this.bool2Real(rightItemCommonData);
                     this.bool2Real(leftItemCommonData);
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     if (leftItemCommonData.type == 10) {
                        if (!leftItemCommonData.isNull && !rightItemCommonData.isNull) {
                           this.real2Decimal(rightItemCommonData);
                           if (rightItemCommonData.cdDecimal.abs().compareTo(new BigDecimal(1.0E-5)) < 0) {
                              leftItemCommonData.cdDecimal = new BigDecimal(0);
                           } else {
                              leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.divide(rightItemCommonData.cdDecimal, 4);
                           }
                        }
                     } else if (Math.abs(rightItemCommonData.cdReal) < 1.0E-5) {
                        leftItemCommonData.cdReal = 0.0;
                     } else {
                        leftItemCommonData.cdReal /= rightItemCommonData.cdReal;
                     }

                     leftItemCommonData.isNull = leftItemCommonData.isNull || rightItemCommonData.isNull;
                     leftItemCommonData.initNullValue();
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 6:
                  default:
                     this.error();
                     continue;
                  case 7:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     if (leftItemCommonData.type == 10) {
                        this.real2Decimal(rightItemCommonData);
                     }

                     leftItemCommonData.cdBool = leftItemCommonData.equals(rightItemCommonData, this.syntaxParser.getBalanceDifferenceValue());
                     leftItemCommonData.type = 5;
                     leftItemCommonData.isNull = false;
                     this.disposeNiporlanItem(rightItem);
                  case 8:
                     continue;
                  case 9:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     if (leftItemCommonData.type == 10) {
                        this.real2Decimal(rightItemCommonData);
                     }

                     leftItem.getCommonData().cdBool = !leftItem.getCommonData().equals(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue());
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().isNull = false;
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 10:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     if (leftItemCommonData.type == 10) {
                        this.real2Decimal(rightItemCommonData);
                     }

                     leftItem.getCommonData().cdBool = leftItem.getCommonData().compareTo(rightItem.getCommonData(), 1.0E-5) > 0;
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().isNull = false;
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 11:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     if (leftItemCommonData.type == 10) {
                        this.real2Decimal(rightItemCommonData);
                     }

                     leftItem.getCommonData().cdBool = leftItem.getCommonData().compareTo(rightItem.getCommonData(), 1.0E-5) < 0;
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().isNull = false;
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 12:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     if (leftItemCommonData.type == 10) {
                        this.real2Decimal(rightItemCommonData);
                     }

                     leftItem.getCommonData().cdBool = leftItem.getCommonData().compareTo(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue()) >= 0;
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().isNull = false;
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 13:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     rightItemCommonData = rightItem.getCommonData();
                     if (rightItemCommonData.type == 10) {
                        this.real2Decimal(leftItemCommonData);
                     }

                     if (leftItemCommonData.type == 10) {
                        this.real2Decimal(rightItemCommonData);
                     }

                     leftItem.getCommonData().cdBool = leftItem.getCommonData().compareTo(rightItem.getCommonData(), this.syntaxParser.getBalanceDifferenceValue()) <= 0;
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().isNull = false;
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 14:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdBool = leftItem.getCommonData().cdBool && rightItem.getCommonData().cdBool;
                     leftItem.getCommonData().isNull = leftItem.getCommonData().isNull && rightItem.getCommonData().isNull;
                     leftItem.getCommonData().initNullValue();
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 15:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdBool = leftItem.getCommonData().cdBool || rightItem.getCommonData().cdBool;
                     leftItem.getCommonData().isNull = leftItem.getCommonData().isNull && rightItem.getCommonData().isNull;
                     leftItem.getCommonData().initNullValue();
                     this.disposeNiporlanItem(rightItem);
                     continue;
                  case 16:
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdBool = !leftItem.getCommonData().cdBool;
                     leftItem.getCommonData().initNullValue();
                     continue;
                  case 17:
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.pop();
                     if (!assignKind) {
                        this.syntaxEvent.dataCell = leftItem.getDataCell();
                        this.syntaxEvent.value = rightItem.getCommonData();
                        if (this.useDataLocator) {
                           this.setCellValue(this.syntaxEvent, leftItem.getDataLocator());
                        } else {
                           this.setCellValue(this.syntaxEvent);
                        }
                     }

                     this.disposeNiporlanItem(rightItem);
                     this.disposeNiporlanItem(leftItem);
                     continue;
               }
            case 1:
               if (item.getFuncName().equals("SUM")) {
                  leftItem = (NiporlanItem)this.stack.peek();
                  this.syntaxEvent.dataCell = leftItem.getDataCell();
                  this.syntaxEvent.value = this.cellValue;
                  this.getStatValue(this.syntaxEvent);
                  leftItem.setType(2);
                  leftItem.getCommonData().assign(this.syntaxEvent.value);
                  if (leftItem.getCommonData().type == 1) {
                     leftItem.getCommonData().type = 2;
                     leftItem.getCommonData().cdReal = (double)leftItem.getCommonData().cdInt;
                  }

                  leftItem.getCommonData().initNullValue();
               } else if (item.getFuncName().equals("L$")) {
                  rightItem = (NiporlanItem)this.stack.pop();
                  leftItem = (NiporlanItem)this.stack.peek();
                  leftItem.getCommonData().cdString = SyntaxParser.asciiCopy(leftItem.getCommonData().cdString, 1, (int)rightItem.getCommonData().cdReal);
                  leftItem.getCommonData().initNullValue();
                  this.disposeNiporlanItem(rightItem);
               } else if (item.getFuncName().equals("R$")) {
                  rightItem = (NiporlanItem)this.stack.pop();
                  leftItem = (NiporlanItem)this.stack.peek();
                  if (SyntaxParser.asciiLength(leftItem.getCommonData().cdString) + 1 - (int)rightItem.getCommonData().cdReal >= 1) {
                     leftItem.getCommonData().cdString = SyntaxParser.asciiCopy(leftItem.getCommonData().cdString, SyntaxParser.asciiLength(leftItem.getCommonData().cdString) + 1 - (int)rightItem.getCommonData().cdReal, (int)rightItem.getCommonData().cdReal);
                  } else {
                     leftItem.getCommonData().cdString = leftItem.getCommonData().cdString;
                  }

                  leftItem.getCommonData().initNullValue();
                  this.disposeNiporlanItem(rightItem);
               } else {
                  NiporlanItem threeItem;
                  if (item.getFuncName().equals("M$")) {
                     threeItem = (NiporlanItem)this.stack.pop();
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdString = SyntaxParser.asciiCopy(leftItem.getCommonData().cdString, (int)rightItem.getCommonData().cdReal, (int)threeItem.getCommonData().cdReal);
                     leftItem.getCommonData().initNullValue();
                     this.disposeNiporlanItem(rightItem);
                     this.disposeNiporlanItem(threeItem);
                  } else if (item.getFuncName().equals("POS")) {
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = (double)SyntaxParser.asciiPos(leftItem.getCommonData().cdString, rightItem.getCommonData().cdString);
                     leftItem.getCommonData().type = 2;
                     leftItem.getCommonData().isNull = false;
                     this.disposeNiporlanItem(rightItem);
                  } else if (item.getFuncName().equals("ABS")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     if (leftItemCommonData.type == 10) {
                        if (!leftItemCommonData.isNull) {
                           leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.abs();
                        }
                     } else {
                        leftItemCommonData.cdReal = Math.abs(leftItem.getCommonData().cdReal);
                     }

                     leftItemCommonData.initNullValue();
                  } else if (item.getFuncName().equals("INT")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     if (leftItemCommonData.type == 10) {
                        if (!leftItemCommonData.isNull) {
                           leftItemCommonData.cdReal = (double)leftItemCommonData.cdDecimal.intValue();
                           leftItemCommonData.type = 2;
                           leftItemCommonData.cdDecimal = null;
                        }
                     } else {
                        leftItemCommonData.cdReal = (double)SyntaxParser.trunc(leftItem.getCommonData().cdReal);
                     }

                     leftItemCommonData.initNullValue();
                  } else if (item.getFuncName().equals("ROUND")) {
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     if (leftItemCommonData.type == 10) {
                        if (!leftItemCommonData.isNull) {
                           leftItemCommonData.cdDecimal = leftItemCommonData.cdDecimal.setScale((int)rightItem.getCommonData().cdReal, 4);
                        }
                     } else {
                        leftItem.getCommonData().cdReal = SyntaxParser.roundFloat(leftItem.getCommonData().cdReal, (int)rightItem.getCommonData().cdReal);
                     }

                     leftItem.getCommonData().initNullValue();
                     this.disposeNiporlanItem(rightItem);
                  } else if (item.getFuncName().equals("MOD")) {
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItemCommonData = leftItem.getCommonData();
                     if (leftItemCommonData.type == 10) {
                        if (!leftItemCommonData.isNull) {
                           leftItemCommonData.cdReal = SyntaxParser.floatMod(leftItemCommonData.cdDecimal.doubleValue(), rightItem.getCommonData().cdReal);
                           leftItemCommonData.type = 2;
                        }
                     } else {
                        leftItem.getCommonData().cdReal = SyntaxParser.floatMod(leftItem.getCommonData().cdReal, rightItem.getCommonData().cdReal);
                     }

                     leftItem.getCommonData().isNull = leftItem.getCommonData().isNull || rightItem.getCommonData().isNull;
                     leftItem.getCommonData().initNullValue();
                     this.disposeNiporlanItem(rightItem);
                  } else if (item.getFuncName().equals("POWER")) {
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = Math.pow(leftItem.getCommonData().cdReal, rightItem.getCommonData().cdReal);
                     leftItem.getCommonData().isNull = leftItem.getCommonData().isNull || rightItem.getCommonData().isNull;
                     leftItem.getCommonData().initNullValue();
                     this.disposeNiporlanItem(rightItem);
                  } else if (item.getFuncName().equals("TAN")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = Math.tan(leftItem.getCommonData().cdReal);
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("ATAN")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = Math.atan(leftItem.getCommonData().cdReal);
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("LN")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = Math.log(leftItem.getCommonData().cdReal);
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("LOG10")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = Math.log(leftItem.getCommonData().cdReal) / Math.log(10.0);
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("LOG")) {
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = Math.log(leftItem.getCommonData().cdReal) / Math.log(rightItem.getCommonData().cdReal);
                     leftItem.getCommonData().isNull = leftItem.getCommonData().isNull || rightItem.getCommonData().isNull;
                     leftItem.getCommonData().initNullValue();
                     this.disposeNiporlanItem(rightItem);
                  } else if (item.getFuncName().equals("VAL")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = SyntaxParser.safeStrToFloat(leftItem.getCommonData().cdString);
                     leftItem.getCommonData().type = 2;
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("STR")) {
                     threeItem = (NiporlanItem)this.stack.pop();
                     rightItem = (NiporlanItem)this.stack.pop();
                     leftItem = (NiporlanItem)this.stack.peek();
                     if (leftItem.getCommonData().type != 1 && leftItem.getCommonData().type != 2) {
                        if (leftItem.getCommonData().type == 6) {
                           leftItem.getCommonData().cdString = leftItem.getCommonData().cdDate == null ? "" : SyntaxParser.datetimeToStr(leftItem.getCommonData().cdDate);
                        } else {
                           leftItem.getCommonData().cdString = leftItem.getCommonData().toString();
                        }
                     } else {
                        leftItem.getCommonData().cdString = SyntaxParser.FloatToStr(leftItem.getCommonData().cdReal, (int)rightItem.getCommonData().cdReal, threeItem.getCommonData().cdBool);
                     }

                     leftItem.getCommonData().type = 3;
                     leftItem.getCommonData().initNullValue();
                     this.disposeNiporlanItem(rightItem);
                     this.disposeNiporlanItem(threeItem);
                  } else if (item.getFuncName().equals("IDC")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdString = String.valueOf(SyntaxParser.idc(leftItem.getCommonData().cdString));
                     leftItem.getCommonData().isNull = false;
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("ISDIGIT")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdBool = SyntaxParser.isDigit(leftItem.getCommonData().cdString);
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().isNull = false;
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("LEN")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdReal = (double)SyntaxParser.asciiLength(leftItem.getCommonData().cdString);
                     leftItem.getCommonData().type = 2;
                     leftItem.getCommonData().isNull = false;
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("ISNULL")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdBool = leftItem.getCommonData().isNull;
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().isNull = false;
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("EXISTCHINESE")) {
                     leftItem = (NiporlanItem)this.stack.peek();
                     leftItem.getCommonData().cdBool = SyntaxParser.existChinese(leftItem.getCommonData().cdString);
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().isNull = false;
                     leftItem.getCommonData().initNullValue();
                  } else if (item.getFuncName().equals("INLIST")) {
                     leftItem = (NiporlanItem)this.stack.pop();
                     int count = (int)leftItem.getCommonData().cdReal;
                     leftItem.getCommonData().type = 5;
                     leftItem.getCommonData().cdBool = false;
                     String[] list = new String[count];

                     int l;
                     for(l = 0; l < count; ++l) {
                        rightItem = (NiporlanItem)this.stack.pop();
                        list[l] = rightItem.getCommonData().cdString;
                        this.disposeNiporlanItem(rightItem);
                     }

                     threeItem = (NiporlanItem)this.stack.pop();

                     for(l = 0; l < count; ++l) {
                        if (list[l].equals(threeItem.getCommonData().cdString) || !list[l].equals("") && threeItem.getCommonData().cdString.indexOf(list[l]) == 0) {
                           leftItem.getCommonData().cdBool = true;
                           break;
                        }
                     }

                     this.disposeNiporlanItem(threeItem);
                     leftItem.getCommonData().isNull = false;
                     leftItem.getCommonData().initNullValue();
                     this.stack.push(leftItem);
                  } else {
                     DictMeaningEvent event;
                     if (item.getFuncName().equals("INDICT")) {
                        leftItem = (NiporlanItem)this.stack.pop();
                        rightItem = (NiporlanItem)this.stack.peek();
                        event = new DictMeaningEvent(this);
                        event.code = rightItem.getCommonData().cdString;
                        event.dictName = leftItem.getCommonData().cdString;
                        this.getDictMeaning(event);
                        rightItem.getCommonData().type = 5;
                        rightItem.getCommonData().cdBool = event.inDict;
                        rightItem.getCommonData().isNull = false;
                        rightItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(leftItem);
                     } else if (item.getFuncName().equals("DICTVALUE")) {
                        threeItem = (NiporlanItem)this.stack.pop();
                        leftItem = (NiporlanItem)this.stack.pop();
                        rightItem = (NiporlanItem)this.stack.peek();
                        event = new DictMeaningEvent(this);
                        event.code = rightItem.getCommonData().cdString;
                        event.dictName = leftItem.getCommonData().cdString;
                        event.meaningName = threeItem.getCommonData().cdString;
                        this.getDictMeaning(event);
                        rightItem.getCommonData().assign(event.meaningValue);
                        if (rightItem.getCommonData().type == 1) {
                           rightItem.getCommonData().type = 2;
                           rightItem.getCommonData().cdReal = (double)rightItem.getCommonData().cdInt;
                        }

                        rightItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(leftItem);
                        this.disposeNiporlanItem(threeItem);
                     } else if (item.getFuncName().equals("UNITSTATE")) {
                        rightItem = (NiporlanItem)this.stack.pop();
                        leftItem = (NiporlanItem)this.stack.peek();
                        UnitStateEvent event = new UnitStateEvent(this, leftItem.getCommonData().cdString, rightItem.getCommonData().cdString);
                        this.getUnitState(event);
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().cdReal = (double)event.state;
                        this.disposeNiporlanItem(rightItem);
                     } else if (item.getFuncName().equals("LINKPLUS")) {
                        threeItem = (NiporlanItem)this.stack.pop();
                        leftItem = (NiporlanItem)this.stack.pop();
                        rightItem = (NiporlanItem)this.stack.peek();
                        LinkPlusEvent event = new LinkPlusEvent(this, rightItem.getDataCell(), (int)leftItem.getCommonData().cdReal, (int)threeItem.getCommonData().cdReal, this.cellValue);
                        this.getLinkPlus(event);
                        rightItem.getCommonData().assign(event.value);
                        if (rightItem.getCommonData().type == 1) {
                           rightItem.getCommonData().type = 2;
                           rightItem.getCommonData().cdReal = (double)rightItem.getCommonData().cdInt;
                        }

                        rightItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(leftItem);
                        this.disposeNiporlanItem(threeItem);
                     } else if (item.getFuncName().equals("FLOATSTAT")) {
                        leftItem = (NiporlanItem)this.stack.peek();
                        FloatStatEvent event = (FloatStatEvent)leftItem.getObject();
                        this.getFloatStat(event);
                        leftItem.setType(2);
                        leftItem.getCommonData().assign(event.value);
                        if (leftItem.getCommonData().type == 1) {
                           leftItem.getCommonData().type = 2;
                           leftItem.getCommonData().cdReal = (double)leftItem.getCommonData().cdInt;
                        }

                        leftItem.getCommonData().initNullValue();
                        leftItem.setObject((Object)null);
                     } else if (item.getFuncName().equals("FLOATCOPY")) {
                        leftItem = (NiporlanItem)this.stack.peek();
                        FloatCopyEvent event = (FloatCopyEvent)leftItem.getObject();
                        this.execFloatCopy(event);
                        leftItem.setType(2);
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().cdReal = (double)event.copyRowCount;
                        leftItem.setObject((Object)null);
                     } else if (item.getFuncName().equals("DATEVALUE")) {
                        rightItem = (NiporlanItem)this.stack.pop();
                        leftItem = (NiporlanItem)this.stack.peek();
                        leftItem.getCommonData().isNull = !SyntaxParser.safeStrToDate(leftItem.getCommonData().getValidcdDate(), leftItem.getCommonData().cdString, rightItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 6;
                        leftItem.getCommonData().initNullValue();
                     } else if (item.getFuncName().equals("FORMATDATE")) {
                        rightItem = (NiporlanItem)this.stack.pop();
                        leftItem = (NiporlanItem)this.stack.peek();
                        leftItem.getCommonData().cdString = SyntaxParser.formatDate(leftItem.getCommonData().getValidcdDate(), rightItem.getCommonData().cdString);
                        leftItem.getCommonData().type = 3;
                        leftItem.getCommonData().initNullValue();
                     } else if (item.getFuncName().equals("DATE")) {
                        threeItem = (NiporlanItem)this.stack.pop();
                        rightItem = (NiporlanItem)this.stack.pop();
                        leftItem = (NiporlanItem)this.stack.peek();
                        SyntaxParser.safeEncodeDate(leftItem.getCommonData().getValidcdDate(), (int)leftItem.getCommonData().cdReal, (int)rightItem.getCommonData().cdReal, (int)threeItem.getCommonData().cdReal);
                        leftItem.getCommonData().type = 6;
                        leftItem.getCommonData().isNull = leftItem.getCommonData().isNull || rightItem.getCommonData().isNull || threeItem.getCommonData().isNull;
                        leftItem.getCommonData().initNullValue();
                        this.disposeNiporlanItem(rightItem);
                        this.disposeNiporlanItem(threeItem);
                     } else if (item.getFuncName().equals("YEAR")) {
                        leftItem = (NiporlanItem)this.stack.peek();
                        leftItem.getCommonData().cdReal = (double)leftItem.getCommonData().getValidcdDate().get(1);
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().initNullValue();
                     } else if (item.getFuncName().equals("MONTH")) {
                        leftItem = (NiporlanItem)this.stack.peek();
                        leftItem.getCommonData().cdReal = (double)(leftItem.getCommonData().getValidcdDate().get(2) + 1);
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().initNullValue();
                     } else if (item.getFuncName().equals("DAY")) {
                        leftItem = (NiporlanItem)this.stack.peek();
                        leftItem.getCommonData().cdReal = (double)leftItem.getCommonData().getValidcdDate().get(5);
                        leftItem.getCommonData().type = 2;
                        leftItem.getCommonData().initNullValue();
                     } else {
                        if (item.getFuncName().equals("TODAY")) {
                           leftItem = this.newNiporlanItem();
                           leftItem.setValue(item.getCommonData());
                           leftItem.getCommonData().type = 6;
                           leftItem.getCommonData().cdDate = new GregorianCalendar();
                           leftItem.getCommonData().cdDate.getTime().setTime(0L);
                           this.stack.push(leftItem);
                           continue;
                        }

                        FunctionListener funcListener = this.syntaxParser.getFunctionListener(item.getFuncName());
                        if (funcListener == null) {
                           this.error("没有找到函数“" + item.getFuncName() + "”的侦听器！");
                           return;
                        }

                        Function func = funcListener.funcByName(item.getFuncName());
                        CommonDatas paramValues = new CommonDatas();
                        NiporlanItem[] niporlanItems;
                        int j;
                        if (!func.isInfiniteParam) {
                           niporlanItems = new NiporlanItem[func.params.size()];

                           for(j = 0; j < func.params.size(); ++j) {
                              niporlanItems[func.params.size() - j - 1] = (NiporlanItem)this.stack.pop();
                           }
                        } else {
                           leftItem = (NiporlanItem)this.stack.pop();
                           niporlanItems = new NiporlanItem[leftItem.getCommonData().cdInt];

                           for(j = 0; j < leftItem.getCommonData().cdInt; ++j) {
                              niporlanItems[leftItem.getCommonData().cdInt - j - 1] = (NiporlanItem)this.stack.pop();
                           }

                           this.disposeNiporlanItem(leftItem);
                        }

                        for(j = 0; j < niporlanItems.length; ++j) {
                           paramValues.add(niporlanItems[j].getCommonData());
                        }

                        leftItem = this.newNiporlanItem();
                        leftItem.setType(2);
                        leftItem.getCommonData().type = func.resultType;
                        leftItem.getCommonData().isNull = false;
                        leftItem.getCommonData().initValue();
                        if (funcListener.callFunction(func.name, paramValues, leftItem.getCommonData(), false) != 0) {
                           leftItem.setValue(leftItem.getCommonData());
                        }

                        if (leftItem.getCommonData().type == 1) {
                           leftItem.getCommonData().type = 2;
                           leftItem.getCommonData().cdReal = (double)leftItem.getCommonData().cdInt;
                        }

                        leftItem.getCommonData().initNullValue();
                        this.stack.push(leftItem);
                     }
                  }
               }
               break;
            case 2:
               leftItem = this.newNiporlanItem();
               leftItem.setValue(item.getCommonData());
               leftItemCommonData = leftItem.getCommonData();
               if (leftItemCommonData.type == 1) {
                  leftItemCommonData.type = 2;
                  leftItemCommonData.cdReal = (double)leftItemCommonData.cdInt;
               }

               this.stack.push(leftItem);
               break;
            case 3:
               leftItem = this.newNiporlanItem();
               leftItem.setValue(item.getDataCell());
               if (this.useDataLocator) {
                  leftItem.setDataLocator(item);
               }

               if (isAssignValue && this.stack.size() == 0) {
                  assignKind = false;
               } else {
                  this.syntaxEvent.dataCell = leftItem.getDataCell();
                  this.syntaxEvent.value = this.cellValue;
                  if (this.useDataLocator) {
                     this.getCellValue(this.syntaxEvent, leftItem.getDataLocator());
                  } else {
                     this.getCellValue(this.syntaxEvent);
                  }

                  leftItem.setType(2);
                  leftItemCommonData = leftItem.getCommonData();
                  leftItemCommonData.assign(this.syntaxEvent.value);
                  if (leftItemCommonData.type == 1) {
                     leftItemCommonData.type = 2;
                     leftItemCommonData.cdReal = (double)leftItemCommonData.cdInt;
                  }

                  if (leftItemCommonData.type == 0) {
                     this.error("Niporlan公式有错误(空数据类型)！");
                  }

                  leftItemCommonData.initNullValue();
               }

               this.stack.push(leftItem);
               break;
            case 4:
               leftItem = this.newNiporlanItem();
               leftItem.setValue(item.getObject());
               this.stack.push(leftItem);
               break;
            default:
               this.error();
         }
      }

   }

   private void getCellValue(SyntaxEvent syntaxEvent) {
      if (this.syntaxParser.getSyntaxListener() != null) {
         this.syntaxParser.getSyntaxListener().getNiValue(syntaxEvent);
      }

   }

   private void getCellValue(SyntaxEvent syntaxEvent, Object dataLocator) {
      if (this.syntaxParser.getOptimizeListener() != null) {
         this.syntaxParser.getOptimizeListener().getCellValue(syntaxEvent, dataLocator);
      }

   }

   private void setCellValue(SyntaxEvent syntaxEvent) {
      if (this.syntaxParser.getSyntaxListener() != null) {
         this.syntaxParser.getSyntaxListener().setNiValue(syntaxEvent);
      }

   }

   private void setCellValue(SyntaxEvent syntaxEvent, Object dataLocator) {
      if (this.syntaxParser.getOptimizeListener() != null) {
         this.syntaxParser.getOptimizeListener().setCellValue(syntaxEvent, dataLocator);
      }

   }

   private void getStatValue(SyntaxEvent syntaxEvent) {
      if (this.syntaxParser.getSyntaxListener() != null) {
         this.syntaxParser.getSyntaxListener().getStatValue(syntaxEvent);
      }

   }

   private void getUnitState(UnitStateEvent unitStateEvent) {
      if (this.syntaxParser.getSyntaxListener() != null) {
         this.syntaxParser.getSyntaxListener().getUnitState(unitStateEvent);
      }

   }

   private void getLinkPlus(LinkPlusEvent linkPlusEvent) {
      if (this.syntaxParser.getSyntaxListener() != null) {
         this.syntaxParser.getSyntaxListener().getLinkPlus(linkPlusEvent);
      }

   }

   private void getFloatStat(FloatStatEvent floatStatEvent) {
      if (this.syntaxParser.getSyntaxListener() != null) {
         this.syntaxParser.getSyntaxListener().getFloatStat(floatStatEvent);
      }

   }

   private void execFloatCopy(FloatCopyEvent floatCopyEvent) {
      if (this.syntaxParser.getSyntaxListener() != null) {
         this.syntaxParser.getSyntaxListener().execFloatCopy(floatCopyEvent);
      }

   }

   private void getDictMeaning(DictMeaningEvent dictMeaningEvent) {
      if (this.syntaxParser.getSyntaxListener() != null) {
         this.syntaxParser.getSyntaxListener().getDictMeaning(dictMeaningEvent);
      }

   }

   private void error() {
      this.error("Niporlan公式有错误 或 不支持的操作！");
   }

   private void error(String message) {
      System.out.println(message);
   }
}
